(**
 * This is the [Example_ocamldoc] module.
 *
 * @author GROSSHANS Nathan
 *)


(**
 * @param e1_p The value to give to the first element of the pair.
 * @param e2_p The value to give to the second element of the pair.
 *)
class ['a] pair (e1_p : 'a) (e2_p : 'a) =
object (self : 'b)
  val mutable e1 = e1_p
  val mutable e2 = e2_p
  method first = e1
  method second = e2
  method private set_first e1_new = e1 <- e1_new
  method private set_second e2_new = e2 <- e2_new
  (**
   * @param p The other pair to compare the current one to.
   *)
  method eq (p : 'b) = p#first = self#first && p#second = self#second
end;;

let p = new pair [1; 2; 3] [4; 5; 6];;
p#first;;
p#second;;
p#eq (new pair [1; 2; 3] [4; 7; 6]);;


class virtual representable =
object
  method virtual string_representation : string
end;;


exception Invalid_scalar;;

(**
 * @param x_p The value to give to the first component of the vector.
 * @param y_p The value to give to the second component of the vector.
 *)
class vector x_p y_p =
object (self)
  inherit [int] pair x_p y_p
  inherit representable
  method x = e1
  method y = e2
  (**
   * @param s The scalar by which to scale the vector.
   * @raise Invalid_scalar When [s] is an invalid scalar.
   *)
  method scale s =
    if (s = infinity) || (s = neg_infinity) || (s = nan)
    then raise Invalid_scalar
    else self#set_first (int_of_float (float_of_int e1 *. s));
         self#set_second (int_of_float (float_of_int e2 *. s))
  method string_representation =
    "Vector: (" ^ string_of_int self#first ^ ", " ^ string_of_int self#second ^
    ")"
end;;

let v = new vector 1 2;;
v#x;;
v#y;;
v#string_representation;;
try
  v#scale infinity
with Invalid_scalar -> print_endline "Scaling by an invalid scalar.";;
v#x;;
v#y;;
v#string_representation;;
v#scale 3.;;
v#x;;
v#y;;
v#string_representation;;
v#scale 0.7;;
v#x;;
v#y;;
v#string_representation;;


class point x_p y_p =
object (self)
  inherit [int] pair x_p y_p
  inherit representable
  method x = self#first
  method y = self#second
  (** @param v The vector by which to move the point. *)
  method move (v : vector) =
    self#set_first (self#first + v#first);
    self#set_second (self#second + v#second)
  method string_representation =
    "Point: (" ^ string_of_int self#first ^ ", " ^ string_of_int self#second ^
    ")"
end;;

let p = new point 7 3;;
p;;
p#x;;
p#y;;
p#string_representation;;
p#move (new vector 1 (-42));;
p#x;;
p#y;;
p#string_representation;;
