(**
 * This is the [Example_ocamldoc] module.
 *
 * @author GROSSHANS Nathan
 *)

(**
 * Polymorphic class of pairs.
 *
 * This is a class that may be used to represent any pair of two elements.
 *)
class ['a] pair : 'a -> 'a ->
object ('b)
  (**
   * Get the first element in the pair.
   * @return The first element in the pair.
   *)
  method first : 'a

  (**
   * Get the second element in the pair.
   * @return The second element in the pair.
   *)
  method second : 'a

  (**
   * Test equality with another pair.
   *
   * Checks if the first element of the pair is equal to the first element of
   * the pair given as argument, and if the second element of the pair is equal
   * to the second element of the pair given as argument.
   * @return [true] if the two pairs are equal, [false] if not. 
   *)
  method eq : 'b -> bool
end;;


(**
 * A virtual representable class.
 *
 * This is a virtual class used to define all the methods needed for classes
 * that may give objects that can be represented.
 *)
class virtual representable :
object
  (**
   * Get a string representation.
   * @return A string representation.
   *)
  method virtual string_representation : string
end;;


(**
 * Exception used to indicate use of an invalid scalar ([infinity],
 * [neg_infinity] and [nan]).
 *)
exception Invalid_scalar;;

(**
 * Class of two-dimensional integral vectors.
 * 
 * This is a class used to represent two-dimensional integral vectors.
 *)
class vector : int -> int ->
object
  (** A vector is a pair. *)
  inherit [int] pair

  (** And is moreover representable. *)
  inherit representable

  (**
   * Get the first component of the vector.
   * @return The first component of the vector.
   *)
  method x : int

  (**
   * Get the second component of the vector.
   * @return The second component of the vector.
   *)
  method y : int

  (**
   * Scale the vector by a scalar.
   *)
  method scale : float -> unit

  (**
   * Get a string representation of the vector.
   * @return A string representation of the vector.
   *)
  method string_representation : string
end;;


(**
 * Class of two-dimensional integral points.
 * 
 * This is a class used to represent two-dimensional integral points.
 *)
class point : int -> int ->
object
  (** A point is a pair. *)
  inherit [int] pair

  (** And is moreover representable. *)
  inherit representable

  (**
   * Get the abscissa of the point.
   * @return The abscissa of the point.
   *)
  method x : int

  (**
   * Get the ordinate of the point.
   * @return The ordinate of the point.
   *)
  method y : int

  (**
   * Move the point by a vector.
   *)
  method move : vector -> unit

  (**
   * Get a string representation of the point.
   * @return A string representation of the point.
   *)
  method string_representation : string
end;;
